@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2016 The OpenMx Project
+#   Copyright 2007-2017 The OpenMx Project
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -129,7 +129,7 @@ growthCurveModel <- mxModel(
 growthCurveFit <- mxRun(growthCurveModel, suppressWarnings=TRUE)
 
 summary(growthCurveFit)
-growthCurveFit$output$estimate
+coef(growthCurveFit)
 
 
 omxCheckCloseEnough(growthCurveFit$output$estimate[["meani"]], 9.930, 0.01)
