@@ -83,6 +83,18 @@ thresholdModelrun <- mxRun(thresholdModel)
 thresholdSaturated <- mxRefModels(thresholdModelrun, run=TRUE)
 summary(thresholdModelrun, refModels=thresholdSaturated)
 
+a <- Sys.time()
+thresholdModelAuto <- mxAutoStart(thresholdModel)
+b <- Sys.time()
+b-a #about 2 seconds on my laptop
+
+thresholdModelAutoRun <- mxRun(thresholdModelAuto)
+
+(b-a) + summary(thresholdModelAutoRun)$wallTime
+summary(thresholdModelrun)$wallTime
+# 37 sec for the automatic start values time plus the estimation time from the auto starts
+# 64 sec for the estimation time from the user starts
+# auto starts provides a net boost in performance
 
 a <- proc.time()
 thresholdModelWLS <- mxModel(thresholdModel, name="WLSThresholdModel", mxDataWLS(ordinalData, type="ULS"), #Change type here!!!
@@ -102,10 +114,14 @@ wls.T <- mxEval(thresholdMatrix, thresholdModelWLSrun) #should be all quants
 ml.L <- mxEval(L, thresholdModelrun) #should be all 0.7
 ml.T <- mxEval(thresholdMatrix, thresholdModelrun) #should be all quants
 
+auto.L <- mxEval(L, thresholdModelAuto)
+auto.T <- mxEval(thresholdMatrix, thresholdModelAuto)
+
 rms <- function(x, y){sqrt(mean((x-y)^2))}
 
 omxCheckTrue(rms(wls.L, .7) < 0.05)
 rms(ml.L, .7)
+omxCheckTrue(rms(ml.L, auto.L) < 0.05)
 
 omxCheckTrue(rms(wls.T, quants) < 0.08)
 rms(ml.T, quants)
@@ -117,7 +133,7 @@ omxCheckWithinPercentError(ml.sum$Chi, wls.sum$Chi, percent=15)
 omxCheckEquals(ml.sum$ChiDoF, wls.sum$ChiDoF)
 
 ciModel <- mxModel(thresholdModelWLSrun, mxCI("L"))
-omxCheckError(mxRun(ciModel, intervals=TRUE), "Confidence intervals are not supported for units 3")
+omxCheckError(mxRun(ciModel, intervals=TRUE), "Confidence intervals are not supported for units r'Wr")
 
 
 #------------------------------------------------------------------------------
@@ -165,7 +181,7 @@ plot(omxGetParameters(trun2), omxGetParameters(wrun2))
 abline(a=0, b=1)
 
 omxCheckCloseEnough(rms(omxGetParameters(trun2), omxGetParameters(wrun2)), 0, .035)
-omxCheckCloseEnough(cor(omxGetParameters(trun2), omxGetParameters(wrun2)), 1, .05)
+omxCheckCloseEnough(cor(omxGetParameters(trun2), omxGetParameters(wrun2)), 1, .06)
 
 
 # new style for model 2
@@ -178,4 +194,17 @@ cbind(omxGetParameters(trun2), omxGetParameters(wrun2), omxGetParameters(wrun2a)
 #  answer.
 omxCheckCloseEnough(omxGetParameters(wrun2), omxGetParameters(wrun2a), 1e-4)
 
+#------------------------------------------------------------------------------
+
+require(OpenMx)
+a <- factor(sample(c('a', 'b', 'c'), size=100, replace=T))
+b <- factor(a, levels=c('a', 'b', 'c', 'd')) #create factor is unused level 'd'
+ma <- mxFactor(a, levels=levels(a))
+mb <- mxFactor(b, levels=levels(b))
+#mb[mb %in% 'c'] <- 'd' #make 'c' the unused level instead of 'd'
+ds <- data.frame(a=ma, b=mb)
+
+omxCheckError(wd <- mxDataWLS(ds), "Variable 'b' has a zero frequency category 'd'.
+Eliminate this level in your mxFactor() or combine categories in some other way.
+Do not pass go. Do not collect $200.")
 
