@@ -30,12 +30,9 @@
 #include "matrix.h"
 #include "omxBuffer.h"
 #include "omxState.h"
-#include "finiteDifferences.h"
 #include <Eigen/Cholesky>
-
-#ifdef SHADOW_DIAG
-#pragma GCC diagnostic warning "-Wshadow"
-#endif
+#include "finiteDifferences.h"
+#include "EnableWarnings.h"
 
 void pda(const double *ar, int rows, int cols);
 
@@ -664,6 +661,7 @@ void FitContext::init()
 	mac = parent? parent->mac : 0;
 	fit = parent? parent->fit : NA_REAL;
 	fitUnits = parent? parent->fitUnits : FIT_UNITS_UNINITIALIZED;
+	skippedRows = 0;
 	est = new double[numParam];
 	infoDefinite = NA_LOGICAL;
 	infoCondNum = NA_REAL;
@@ -772,6 +770,7 @@ void FitContext::updateParent()
 	parent->wanted |= wanted;
 	parent->fit = fit;
 	parent->fitUnits = fitUnits;
+	parent->skippedRows = skippedRows;
 	parent->mac = mac;
 	parent->infoDefinite = infoDefinite;
 	parent->infoCondNum = infoCondNum;
@@ -877,7 +876,7 @@ void FitContext::resetIterationError()
 
 void FitContext::recordIterationError(const char* msg, ...)
 {
-	// could record the parameter vector here for a better error message TODO
+	// Can avoid overhead of setting error if one is already set TODO
 	va_list ap;
 	va_start(ap, msg);
 	IterationError = string_vsnprintf(msg, ap);
@@ -921,7 +920,7 @@ static void _fixSymmetry(const char *name, double *mat, size_t numParam, bool fo
 
 static void omxRepopulateRFitFunction(omxFitFunction* oo, double* x, int n)
 {
-	omxRFitFunction* rFitFunction = (omxRFitFunction*)oo->argStruct;
+	omxRFitFunction* rFitFunction = (omxRFitFunction*)oo;
 
 	SEXP theCall, estimate;
 
@@ -948,6 +947,7 @@ static void omxRepopulateRFitFunction(omxFitFunction* oo, double* x, int n)
 
 void FitContext::ensureParamWithinBox(bool nudge)
 {
+	if (OMX_DEBUG) mxLog("FitContext::ensureParamWithinBox(nudge=%d)", nudge);
 	for (size_t px = 0; px < varGroup->vars.size(); ++px) {
 		omxFreeVar *fv = varGroup->vars[px];
 		if (nudge && est[px] == 0.0) {
@@ -972,17 +972,6 @@ void copyParamToModelInternal(FreeVarGroup *varGroup, omxState *os, double *at)
 {
 	size_t numParam = varGroup->vars.size();
 
-	if(OMX_DEBUG) {
-		std::string buf;
-		buf += string_snprintf("copyParamToModel: c(");
-		for(size_t k = 0; k < numParam; k++) {
-			buf += string_snprintf("%g", at[k]);
-			if (k < numParam-1) buf += string_snprintf(", ");
-		}
-		buf += (")\n");
-		mxLogBig(buf);
-	}
-
 	for(size_t k = 0; k < numParam; k++) {
 		omxFreeVar* freeVar = varGroup->vars[k];
 		freeVar->copyToState(os, at[k]);
@@ -1676,7 +1665,8 @@ static const struct omxComputeTableEntry omxComputeTable[] = {
 	{"MxComputeReportDeriv", &newComputeReportDeriv},
 	{"MxComputeConfidenceInterval", &newComputeConfidenceInterval},
 	{"MxComputeReportExpectation", &newComputeReportExpectation},
-	{"MxComputeTryHard", &newComputeTryHard}
+	{"MxComputeTryHard", &newComputeTryHard},
+	{"MxComputeNelderMead", &newComputeNelderMead}
 };
 
 omxCompute *omxNewCompute(omxState* os, const char *type)
@@ -1867,7 +1857,6 @@ void ComputeEM::initFromFrontend(omxState *globalState, SEXP rObj)
 	for (int wx=0; wx < Rf_length(slotValue); ++wx) {
 		int objNum = INTEGER(slotValue)[wx];
 		omxExpectation *expectation = globalState->expectationList[objNum];
-		setFreeVarGroup(expectation, varGroup);
 		omxCompleteExpectation(expectation);
 		expectations.push_back(expectation);
 	}
@@ -1889,7 +1878,6 @@ void ComputeEM::initFromFrontend(omxState *globalState, SEXP rObj)
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("observedFit")));
 	fit3 = globalState->algebraList[ INTEGER(slotValue)[0] ];
 	if (fit3->fitFunction) {
-		setFreeVarGroup(fit3->fitFunction, varGroup);
 		omxCompleteFitFunction(fit3);
 	}
 
@@ -2177,6 +2165,7 @@ void ComputeEM::computeImpl(FitContext *fc)
 			FitContext *fc1 = new FitContext(fc, fit1->varGroup);
 			int startIter = fc1->iterations;
 			fit1->compute(fc1);
+			fc1->wanted &= ~FF_COMPUTE_HESSIAN;  // discard garbage
 			mstepIter = fc1->iterations - startIter;
 			totalMstepIter += mstepIter;
 			mstepInform = fc1->getInform();
@@ -2260,7 +2249,7 @@ void ComputeEM::computeImpl(FitContext *fc)
 	if (verbose >= 1) mxLog("ComputeEM: cycles %d/%d total mstep %d fit %f inform %d",
 				EMcycles, maxIter, totalMstepIter, bestFit, fc->getInform());
 
-	if (!converged || information == EMInfoNone) return;
+	if (!converged || fc->skippedRows || information == EMInfoNone) return;
 
 	optimum.resize(freeVars);
 	memcpy(optimum.data(), fc->est, sizeof(double) * freeVars);
@@ -2634,12 +2623,10 @@ void omxComputeOnce::initFromFrontend(omxState *globalState, SEXP rObj)
 		if (objNum >= 0) {
 			omxMatrix *algebra = globalState->algebraList[objNum];
 			if (!algebra->fitFunction) Rf_error("ComputeOnce can only handle fit functions");
-			setFreeVarGroup(algebra->fitFunction, varGroup);
 			omxCompleteFitFunction(algebra);
 			algebras.push_back(algebra);
 		} else {
 			omxExpectation *expectation = globalState->expectationList[~objNum];
-			setFreeVarGroup(expectation, varGroup);
 			omxCompleteExpectation(expectation);
 			expectations.push_back(expectation);
 		}
@@ -2912,296 +2899,10 @@ void ComputeReportExpectation::reportResults(FitContext *fc, MxRList *, MxRList
 		omxExpectationRecompute(fc, curExpectation);
 		SEXP rExpect;
 		Rf_protect(rExpect = Rf_allocVector(LGLSXP, 1)); // placeholder to attach attributes
-		if(curExpectation->populateAttrFun != NULL) {
-			if(OMX_DEBUG) { mxLog("Expectation %d has attribute population.", (int) index); }
-			curExpectation->populateAttrFun(curExpectation, rExpect);
-		}
+		if(OMX_DEBUG) { mxLog("Expectation %d has attribute population.", (int) index); }
+		curExpectation->populateAttr(rExpect);
 		SET_VECTOR_ELT(expectations, index, rExpect);
 	}
 
 	result->add("expectations", expectations);
 }
-
-// ------------------------------------------------------------
-
-void GradientOptimizerContext::copyBounds()
-{
-	FreeVarGroup *varGroup = fc->varGroup;
-	int px=0;
-	for (size_t vx=0; vx < fc->profiledOut.size(); ++vx) {
-		if (fc->profiledOut[vx]) continue;
-		solLB[px] = varGroup->vars[vx]->lbound;
-		if (!std::isfinite(solLB[px])) solLB[px] = NEG_INF;
-		solUB[px] = varGroup->vars[vx]->ubound;
-		if (!std::isfinite(solUB[px])) solUB[px] = INF;
-		++px;
-	}
-}
-
-void GradientOptimizerContext::setupSimpleBounds()
-{
-	solLB.resize(numFree);
-	solUB.resize(numFree);
-	copyBounds();
-}
-
-void GradientOptimizerContext::setupIneqConstraintBounds()
-{
-	solLB.resize(numFree);
-	solUB.resize(numFree);
-	copyBounds();
-
-	omxState *globalState = fc->state;
-	int eqn, nineqn;
-	globalState->countNonlinearConstraints(eqn, nineqn, false);
-	equality.resize(eqn);
-	inequality.resize(nineqn);
-};
-
-void GradientOptimizerContext::setupAllBounds()
-{
-	omxState *st = fc->state;
-	int n = (int) numFree;
-
-	// treat all constraints as non-linear
-	int eqn, nineqn;
-	st->countNonlinearConstraints(eqn, nineqn, false);
-	int ncnln = eqn + nineqn;
-	solLB.resize(n + ncnln);
-	solUB.resize(n + ncnln);
-
-	copyBounds();
-
-	int index = n;
-	for(int constraintIndex = 0; constraintIndex < int(st->conListX.size()); constraintIndex++) {
-		omxConstraint &cs = *st->conListX[constraintIndex];
-		omxConstraint::Type type = cs.opCode;
-		switch(type) {
-		case omxConstraint::LESS_THAN:
-		case omxConstraint::GREATER_THAN:
-			for(int offset = 0; offset < cs.size; offset++) {
-				solLB[index] = NEG_INF;
-				solUB[index] = -0.0;
-				index++;
-			}
-			break;
-		case omxConstraint::EQUALITY:
-			for(int offset = 0; offset < cs.size; offset++) {
-				solLB[index] = -0.0;
-				solUB[index] = 0.0;
-				index++;
-			}
-			break;
-		default:
-			Rf_error("Unknown constraint type %d", type);
-		}
-	}
-}
-
-void GradientOptimizerContext::reset()
-{
-	feasible = false;
-	bestFit = std::numeric_limits<double>::max();
-	eqNorm = 0;
-	ineqNorm = 0;
-}
-
-GradientOptimizerContext::GradientOptimizerContext(FitContext *_fc, int _verbose)
-	: fc(_fc), verbose(_verbose)
-{
-	numFree = 0;
-	for (size_t vx=0; vx < fc->profiledOut.size(); ++vx) {
-		if (fc->profiledOut[vx]) continue;
-		++numFree;
-	}
-	optName = "?";
-	fitMatrix = NULL;
-	ControlMinorLimit = 800;
-	ControlRho = 1.0;
-	ControlTolerance = nan("uninit");
-	useGradient = false;
-	warmStart = false;
-	ineqType = omxConstraint::LESS_THAN;
-	avoidRedundentEvals = false;
-	est.resize(numFree);
-	grad.resize(numFree);
-	copyToOptimizer(est.data());
-	numOptimizerThreads = (fc->childList.size() && !fc->openmpUser)? fc->childList.size() : 1;
-	CSOLNP_HACK = false;
-	reset();
-}
-
-double GradientOptimizerContext::recordFit(double *myPars, int* mode)
-{
-	double fit = solFun(myPars, mode);
-	if (std::isfinite(fit) && fit < bestFit) {
-		bestFit = fit;
-		Eigen::Map< Eigen::VectorXd > pvec(myPars, fc->numParam);
-		bestEst = pvec;
-	}
-	return fit;
-}
-
-bool GradientOptimizerContext::hasKnownGradient() const
-{
-	return fc->ciobj && fc->ciobj->gradientKnown();
-}
-
-void GradientOptimizerContext::copyToOptimizer(double *myPars)
-{
-	int px=0;
-	for (size_t vx=0; vx < fc->profiledOut.size(); ++vx) {
-		if (fc->profiledOut[vx]) continue;
-		myPars[px] = fc->est[vx];
-		++px;
-	}
-}
-
-void GradientOptimizerContext::useBestFit()
-{
-	fc->fit = bestFit;
-	est = bestEst;
-	// restore gradient too? TODO
-}
-
-void GradientOptimizerContext::copyFromOptimizer(double *myPars, FitContext *fc2)
-{
-	int px=0;
-	for (size_t vx=0; vx < fc2->profiledOut.size(); ++vx) {
-		if (fc2->profiledOut[vx]) continue;
-		fc2->est[vx] = myPars[px];
-		++px;
-	}
-	fc2->copyParamToModel();
-}
-
-void GradientOptimizerContext::finish()
-{
-	fc->grad.resize(fc->numParam);
-	fc->grad.setConstant(nan("unset"));
-
-	int px=0;
-	for (size_t vx=0; vx < fc->profiledOut.size(); ++vx) {
-		if (fc->profiledOut[vx]) continue;
-		fc->est[vx] = est[px];
-		fc->grad[vx] = grad[px];
-		++px;
-	}
-	fc->copyParamToModel();
-}
-
-double GradientOptimizerContext::evalFit(double *myPars, int thrId, int *mode)
-{
-	FitContext *fc2 = thrId >= 0? fc->childList[thrId] : fc;
-	Eigen::Map< Eigen::VectorXd > Est(myPars, fc2->numParam);
-	copyFromOptimizer(myPars, fc2);
-	int want = FF_COMPUTE_FIT;
-	ComputeFit(optName, fc2->lookupDuplicate(fitMatrix), want, fc2);
-	if (fc2->outsideFeasibleSet() || isErrorRaised()) {
-		*mode = -1;
-	}
-	if (verbose >= 3) {
-		mxLog("fit %f (mode %d)", fc2->fit, *mode);
-	}
-	return fc2->fit;
-}
-
-double GradientOptimizerContext::solFun(double *myPars, int* mode)
-{
-	Eigen::Map< Eigen::VectorXd > Est(myPars, fc->numParam);
-	if (feasible && avoidRedundentEvals && *mode == prevMode) {
-		if (Est == prevPoint) {
-			return fc->fit;
-		}
-	}
-
-	if (*mode == 1) {
-		fc->iterations += 1;
-		Global->reportProgress("MxComputeGradientDescent", fc);
-	}
-	copyFromOptimizer(myPars, fc);
-
-	int want = FF_COMPUTE_FIT;
-	// eventually want to permit analytic gradient during CI
-	if (*mode > 0 && !fc->ciobj && useGradient && fitMatrix->fitFunction->gradientAvailable) {
-		fc->grad.resize(fc->numParam);
-		fc->grad.setZero();
-		want |= FF_COMPUTE_GRADIENT;
-	}
-	ComputeFit(optName, fitMatrix, want, fc);
-
-	if (fc->outsideFeasibleSet() || isErrorRaised()) {
-		*mode = -1;
-	} else {
-		feasible = true;
-		if (avoidRedundentEvals) {
-			prevPoint = Est;
-			prevMode = *mode;
-		}
-		if (want & FF_COMPUTE_GRADIENT) {
-			int px=0;
-			for (size_t vx=0; vx < fc->profiledOut.size(); ++vx) {
-				if (fc->profiledOut[vx]) continue;
-				grad[px++] = fc->grad[vx];
-			}
-		}
-	}
-
-	if (verbose >= 3) {
-		mxLog("fit %f (mode %d)", fc->fit, *mode);
-	}
-
-	return fc->fit;
-};
-
-// NOTE: All non-linear constraints are applied regardless of free
-// variable group.
-void GradientOptimizerContext::solEqBFun()
-{
-	const int eq_n = (int) equality.size();
-	omxState *st = fc->state;
-
-	if (!eq_n) return;
-
-	int cur = 0;
-	for(int j = 0; j < int(st->conListX.size()); j++) {
-		omxConstraint &con = *st->conListX[j];
-		if (con.opCode != omxConstraint::EQUALITY) continue;
-
-		con.refreshAndGrab(fc, &equality(cur));
-		cur += con.size;
-	}
-
-	if (verbose >= 3) {
-		mxPrintMat("equality", equality);
-	}
-};
-
-// NOTE: All non-linear constraints are applied regardless of free
-// variable group.
-void GradientOptimizerContext::myineqFun()
-{
-	const int ineq_n = (int) inequality.size();
-	omxState *st = fc->state;
-
-	if (!ineq_n) return;
-
-	int cur = 0;
-	for (int j = 0; j < int(st->conListX.size()); j++) {
-		omxConstraint &con = *st->conListX[j];
-		if (con.opCode == omxConstraint::EQUALITY) continue;
-
-		con.refreshAndGrab(fc, (omxConstraint::Type) ineqType, &inequality(cur));
-		cur += con.size;
-	}
-
-	if (CSOLNP_HACK) {
-		// CSOLNP doesn't know that inequality constraints can be inactive TODO
-	} else {
-		inequality = inequality.array().max(0.0);
-	}
-
-	if (verbose >= 3) {
-		mxPrintMat("inequality", inequality);
-	}
-};
