@@ -100,5 +100,8 @@ void omxElementDbinom(omxMatrix** args, int numArgs, omxMatrix* result);
 void omxElementPbinom(omxMatrix** args, int numArgs, omxMatrix* result);
 void omxElementDcauchy(omxMatrix** args, int numArgs, omxMatrix* result);
 void omxElementPcauchy(omxMatrix** args, int numArgs, omxMatrix* result);
+void omxRowSums(omxMatrix** args, int numArgs, omxMatrix* result);
+void omxColSums(omxMatrix** args, int numArgs, omxMatrix* result);
+void evaluateOnGrid(omxMatrix** args, int numArgs, omxMatrix* result);
 extern const omxAlgebraTableEntry omxAlgebraSymbolTable[];
 #endif
